/*
 * arm7tdmi.h : the header file for ARM7TDMI core
 * 
 * Copyright (C) 2003, Rongkai zhan <zhanrk@163.com>, All rights reserved.
 *
 * $Id: arm7tdmi.h,v 1.1.1.1 2004/09/26 03:59:34 zhanrk Exp $
 */

#ifndef JTAGER_ARM7TDMI_H
#define JTAGER_ARM7TDMI_H

/* The Test data register lenght of ARM7TDMI */
#define ARM7TDMI_BYPASS_REG_BITNR 	1
#define ARM7TDMI_IDCODE_REG_BITNR 	32
#define ARM7TDMI_INSTRUCTION_REG_BITNR 	4
#define ARM7TDMI_SCANPATH_REG_BITNR 	4

/* the bit length of all scan chains for ARM7TDMI core */
#define ARM7TDMI_SCANCHAIN0_BITNR 	113
#define ARM7TDMI_SCANCHAIN1_BITNR 	33
#define ARM7TDMI_SCANCHAIN2_BITNR 	38

/*
 * the EmbeddedICE-RT logic of ARM7TDMI
 */

/* ARM7TDMI EmbeddedICE-RT logic's all registers address definition */
#define ARM7TDMI_ICE_DBGCTL	0x00 /* debug control register */
#define ARM7TDMI_ICE_DBGSTAT	0x01 /* debug status register */
#define ARM7TDMI_ICE_ABTSTAT	0x02 /* Abort status register */
#define ARM7TDMI_ICE_DCC_CTL	0x04
#define ARM7TDMI_ICE_DCC_DATA	0x05
#define ARM7TDMI_ICE_WP0_ADDRVALUE	0x08
#define ARM7TDMI_ICE_WP0_ADDRMASK	0x09
#define ARM7TDMI_ICE_WP0_DATAVALUE	0x0A
#define ARM7TDMI_ICE_WP0_DATAMASK	0x0B
#define ARM7TDMI_ICE_WP0_CTLVALUE	0x0C
#define ARM7TDMI_ICE_WP0_CTLMASK	0x0D
#define ARM7TDMI_ICE_WP1_ADDRVALUE	0x10
#define ARM7TDMI_ICE_WP1_ADDRMASK	0x11
#define ARM7TDMI_ICE_WP1_DATAVALUE	0x12
#define ARM7TDMI_ICE_WP1_DATAMASK	0x13
#define ARM7TDMI_ICE_WP1_CTLVALUE	0x14
#define ARM7TDMI_ICE_WP1_CTLMASK	0x15

#define ARM_NOP	0xE1A00000 /* NOP instruction for ARM core */

extern target_t arm7tdmi_target;

/* from ice.c */
extern int arm7tdmi_ice_read(u32 reg_addr, u32 *reg_val);
extern int arm7tdmi_ice_write(u32 reg_addr, u32 reg_val);
extern int arm7tdmi_ice_set_breakpt(void);
extern int arm7tdmi_ice_clear_breakpt(void);

/* from arm7tdmi.c */
extern int arm7tdmi_setup(void);
extern int arm7tdmi_halt(void);
extern int arm7tdmi_exec_instruction(u32 *writein, u32 *readout);
extern int arm7tdmi_restart(u32 pc);

/* from register.c */
extern u32 arm7tdmi_r0_read(void);
extern void arm7tdmi_r0_write(u32 newval);
extern int arm7tdmi_get_core_state(void);
extern int arm7tdmi_register_read(core_register_t *reg);
extern int arm7tdmi_register_write(core_register_t *reg, u32 value);

/* from memory.c */
extern int arm7tdmi_memory_read8(u8 *buf, u32 address, u32 length);
extern int arm7tdmi_memory_write8(u8 *buf, u32 address, u32 length);
extern int arm7tdmi_memory_read16(u16 *buf, u32 address, u32 length);
extern int arm7tdmi_memory_write16(u16 *buf, u32 address, u32 length);
extern int arm7tdmi_memory_read32(u32 * buf, u32 address, u32 length);
extern int arm7tdmi_memory_write32(u32 * buf, u32 address, u32 length);

#endif /* !JTAGER_ARM7TDMI_H */
