/*
 * command.h : the header file for command line parser
 *
 * Copyright (C) 2003, Rongkai Zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: command.h,v 1.1.1.1 2004/09/26 03:59:34 zhanrk Exp $ */

#ifndef JTAGER_COMMAND_H
#define JTAGER_COMMAND_H

struct option_desc {
	const char *name;
	const char *desc;
};

#ifdef cmd_t
#undef cmd_t
#endif

/* command descriptor */
typedef struct cmd_struct {
	/* 'name' and 'desc' are required */
	const char *name;
	const char *desc;   /* simple command description info */

	const char *alias; /* command alias */
	
	/* 'usage' and 'verbose' are optional, and may be NULL */
	const char *usage; /* command usage info */
	const char *verbose; /* verbose command description info */
	
	struct option *long_options; /* the GNU-style long options */
	struct option_desc *optdesc; /* The description of each option */
	int non_option_args; /* the number of non-option arguments */

	//void (*help) (void);
	int (*run) (int argc, char **argv);
}cmd_t;

/* All commads table */
extern cmd_t *cmds_tbl[];

extern cmd_t *command_index(const char *cmdname);

#define cmd_index(cmdname) 	command_index((cmdname))

#define PAGE_SIZE 4096

extern int do_help(cmd_t *cmd);

/*
 * Memory block size indicator
 * for example:
 *   - 100b/100B : 100 bytes.
 *   - 100s/100S : 100 short words (half-words), or 200 bytes
 *   - 100w/100W : 100 words, or 200 half-words, or 400 bytes
 */
#define BYTE_INDICATOR     'b'
#define HALFWORD_INDICATOR 's'
#define WORD_INDICATOR     'w'

#define BYTE_FMT	1
#define HALFWORD_FMT	2
#define WORD_FMT 	4

/*
 * Return value: BYTE_FMT     - byte
 *               HALFWORD_FMT - halfword
 *               WORD_FMT     - word
 *               -1           - unkown size format, or bad size string
 */
int get_memsize(char *size_str, u32 *size);

#define addr32_align(addr) ((addr) & 0xFFFFFFFC)
#define addr16_align(addr) ((addr) & 0xFFFFFFFE)

int print_syntax_error(char *cmdname);
int print_arg_error(char *cmdname, char *invalid_arg);

/*
 * NOTE: This function is the only entry point of the commands parser.
 */
extern int parse_cmdline(char * cmdline);

int ask_yesno(const char *prompt);

#define SIZE_OPTDESC \
	{"size", "The size of the memory block. The default value: 1 word(32-bit)." \
	 "\n\t\tWe can append a 'b' 'B' 's' 'S' 'w' or 'W' charater to the end of the " \
	 "\n\t\tvalue of --size option, to specify how many bytes, how many half-words,"\
	 "\n\t\tor how many words. For example, \"--size=0x1bb\" means 27 (=0x1b) bytes."\
	 "\n\t\t\"--size=0x1bs\" means 27 half-words, \"--size=0x1bw\" means 27 words," \
	 "\n\t\twhile \"--size=20\" means 20 words by default."}
	
#endif /* JTAGER_COMMAND_H */
