/*
 * jtager.h: the header file for JTAGER
 *
 * Copyright (C) 2003, Rongkai Zhan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: jtager.h,v 1.3 2004/10/17 13:54:32 zhanrk Exp $ */

#ifndef JTAGER_H
#define JTAGER_H

#include <getopt.h>
#include "types.h"

#define PACKAGE		"JTAGER"
#define VERSION		"0.3.0"
#define AUTHOR		"Rongkai Zhan"
#define AUTHOR_CHINESE		"ղٿ"	/* my chinese name:-) */

#define JTAGER_DEBUG

extern const char *jtager_history;

/* jtager error number codes */
#define ERR_NONE		0
#define ERR_COMMON 	1 /* any errors */
#define ERR_INVALID_PARAM 	2 /* invalid function paramter */

#define ERR_JTAG_INSTRUCTION_FAILED 3 /* failed to r/w instruction register */
/* read an invalid idcode value from IDCODE register */
#define ERR_JTAG_INVALID_IDCODE 4
/* the specified scan path is non-existed */
#define ERR_JTAG_NONEXISTED_SCANPATH 5
#define ERR_JTAG_SCANPATH_FAILED 6 /* falied to r/w scan path register */

/* the specified ICE register is non-existed */
#define ERR_TARGET_NONEXISTED_ICE_REG 7
#define ERR_TARGET_HALT_FAILED 8 /* failed to halt the target */
#define ERR_TARGET_IS_RUNNING 9 /* the target is running */
#define ERR_TARGET_IN_THUMB_MODE 10
#define ERR_TARGET_UNKNOWN_REG 11
#define ERR_TARGET_UNKNOWN_ICE_REG 12
#define ERR_TARGET_INVALID_CPSR 13 /* invalid CPSR value */
#define ERR_TARGET_NO_SPSR 14 /* no SPSR register in user mode */

#define ERR_FLASH_TIMEOUT 15

extern char *err_str(int retval);

/* from misc.c */
extern void reverse_bit_order(int bitnr, u8 *input, u8 *output);
extern int str2int(const char *strval, u32 *value);

extern int setup_config(void);

#include "command.h"

#endif /* !JTAGER_H */
