/*
 * cmd_execbat.c : implements the execbat command
 *
 * Copyright (C) 2004, Rongkai Zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: cmd_execbat.c,v 1.1.1.1 2004/09/26 03:59:44 zhanrk Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <sys/stat.h>

#include "jtager.h"
#include "target.h"

/*
 * A jtager batch-command file should begin from this line
 */
static char *batfile_header = "#!jtager-batch-commands";

static char *batfile = NULL;

cmd_t execbat_cmd;

static int do_execbat(FILE *fp)
{
	char *buf = NULL, *cur_line, *tmp;
	int linecnt = 0, ret, len;
	
	while (1) {
		if (buf)
			free(buf); /* free the buffer allocated by getline() */
		buf = NULL;
		len = 0;
		linecnt++;
		
		/* read a line */
		len = getline(&buf, &len, fp);
		if (len == -1) {
			if (buf)
				free(buf);
			break; /* an error occurs or end of file is reached */
		}
		
		/* strip the ending newline character */
		if (buf[len-1] =='\n')
			buf[len-1] = '\0';
		if (buf[0] == '\0')
			continue; /* skip null line */
		
		cur_line = buf;
		
		/* translate the tab into a space character */
		while (1) {
			tmp = strchr(cur_line, '\t');
			if (tmp)
				*tmp = ' ';
			else
				break;
		}
		
		/* strip the leading space */
		while (*cur_line == ' ')
			cur_line++;
		
		/* a comment line that is leading with the '#' character ? */
		if (*cur_line == '#')
			continue;
		
		/* discard the comment string within a line */
		tmp = strchr(cur_line, '#');
		if (tmp)
			*tmp = '\0';
		
		if (*cur_line == '\0')
			continue;
		
		/* skip ourself */
		ret = strncasecmp(cur_line, execbat_cmd.name, strlen(execbat_cmd.name));
		if (ret == 0)
			continue;
		
		printf("*** line %d: %s\n", linecnt, cur_line);
		parse_cmdline(cur_line);		
	} /* end of while (1) */
	
	return ret;
} /* end of do_execbat() */

static int execbat_run(int argc, char **argv)
{
	struct stat batfile_attr;
	FILE *fp;
	char buf[50];
	int i, ret;
	
	if (argc == 1)
		return do_help(&execbat_cmd);
	
	if (argc != 2)
		goto syntax_err;
	
	batfile = argv[1];
	ret = stat(batfile, &batfile_attr);
	if (ret)
		goto batfile_err;
	
	fp = fopen(batfile, "r");
	if (!fp)
		goto batfile_err;

	memset(buf, 0, 50);
	for(i = 0; i < strlen(batfile_header); i++)
		buf[i] = fgetc(fp);
	ret = strcasecmp(buf, batfile_header);
	if (ret)
		goto bad_batfile;
	fseek(fp, 0, SEEK_SET);
	
	if (target->status == TARGET_STATUS_RUNNING)
		goto target_err;
	
	/* go */
	ret = do_execbat(fp);
	
	fclose(fp);
	return ret;
syntax_err:
	return print_syntax_error(argv[0]);
batfile_err:
	printf("%s: ", execbat_cmd.name);
	perror(batfile);
	return ret;
bad_batfile:
	printf("%s: %s: It is not a valid jtager batch-command file.\n", execbat_cmd.name, batfile);
	return -1;
target_err:
	printf("error: %s\n", err_str(-ERR_TARGET_IS_RUNNING));
	return -1;
} /* end of execbat_run() */

cmd_t execbat_cmd = {
	.name = "execbat",
	.desc = "Execute a jtager batch-command file.",
	.alias = NULL,
	.usage = "execbat /your/jtager/batch/file",
	.verbose = NULL,
	.long_options = NULL,
	.optdesc = NULL,
	.non_option_args = 1,

	.run  = execbat_run,
};
