/*
 * cmdline/cmd_restart.c: implements the JTAGER restart command
 *
 * Copyright (C) 2004, Rongkai Zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: cmd_restart.c,v 1.1.1.1 2004/09/26 03:59:44 zhanrk Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jtager.h"
#include "jtag.h"
#include "target.h"

static char *pc_str = NULL; /* --pc option */
static char *restart_short_options = "p";
static struct option restart_options[] = {
	{"pc", required_argument, 0, 'p'},
	{0, 0, 0, 0}
};

static struct option_desc restart_optdesc[] = {
	{"pc", "Which position do you want the target to restart"},
	{NULL, NULL}
};

cmd_t restart_cmd;

static int restart_run(int argc, char **argv)
{
	int option_index = 0, ret = 0;
	unsigned long pc;
	char *invalid_arg = NULL;

	/*
	 * Step 1: reset all option flag variables and option value strings
	 */
	pc_str = NULL;
	
	/*
	 * Step 2: decode command options and non-option arguments
	 */
	while (1) {
		ret = getopt_long(argc, argv, restart_short_options,
				restart_options, &option_index);

		/* detect the end of the options */
		if (ret == -1) {
			ret = 0;
			break;
		}
		
		switch (ret) {
		case 0:
			/* this option has set a flag, do nothing else now */
			if (restart_options[option_index].flag != 0)
				break;
			else
				goto syntax_err;
		case 'p':
			/* --pc option */
			if (pc_str)
				goto syntax_err;
			pc_str = optarg;
			break;
		case '?':
			/* Fall through to default ...
			 * But before we print our syntax error message,
			 * getopt_long will print an error message, first.
			 */
		default:
			ret = -1;
			goto syntax_err;
		} /* end of switch (...) */
	} /* end of while (1) */
	
	/* Try to get non-option argument */
	if (optind < argc) {
		if (pc_str)
			goto syntax_err;
		pc_str = (char *)argv[argc-1];
	}
	
	/*
	 * Step 3 - Translate command parameters
	 */
	pc = 0x0; /* default value: 0 */
	if (pc_str) {
		invalid_arg = pc_str;
		ret = str2int(pc_str, &pc);
		if (ret)
			goto arg_err;
		pc = addr32_align(pc);
	}
	
	/*
	 * Step 4: go, go, go ...
	 */
	printf("Restarting the target processor at 0x%.8lX\n", pc);
	t_op->restart(pc);
	
	return 0;
syntax_err:
	return print_syntax_error(argv[0]);
arg_err:
	return print_arg_error(argv[0], invalid_arg);
}
	
cmd_t restart_cmd = {
	.name = "restart",
	.desc = "Restart the halted target from some position",
	.alias = NULL,
	.usage = "restart [[--pc=]dest]",
	.verbose = NULL,
	.long_options = restart_options,
	.optdesc = restart_optdesc,
	.non_option_args = 1,

	.run = restart_run
};
