/*
 * conf/config.c : parse the config file - /etc/jtager/jtager.conf
 *
 * Copyright (C) 2003, Rongkai Zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: config.c,v 1.2 2004/10/17 13:54:32 zhanrk Exp $ */

/*
 * Reference:
 * the sample.c file in the source tarball "parsecfg-3.6.7.tar.gz",
 * see http://www.icewalkers.com/Linux/Software/56270/parsecfg.html.
 */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#include "jtager.h"
#include "target.h"
#include "jtag.h"
#include "parsecfg.h"

/* the full path name of JTAGER config file */
static const char *conf_filename = "/etc/jtager/jtager.conf";

/* the paramters within the [TARGET] section */
char **core_type; /* the "core" option */

/* the parameters within the [JTAG EMULATOR] section */
char **emulator_type; /* the "emulator" option */

/* array of cfgStruct */
cfgStruct jtager_cfg[] = {
	/* parameter  type  address of variable */
	{"core",     CFG_STRING, &core_type},
	{"emulator", CFG_STRING, &emulator_type},
	{NULL,       CFG_END, NULL}
};

/**
 * setup_config - the entry for parsing /etc/jtager/jtager.conf
 *
 * Return value: if success, return 0. Otherwise return -1.
 */
int setup_config(void)
{
	struct stat st;
	int max, retval = -1;
	
	/*
	 * Before reading the config file /etc/jtager/jtager.conf,
	 * we stat its file attributes.
	 */
	retval = stat(conf_filename, &st);
	if (retval) {
		perror(conf_filename);
		return retval;
	}

	if (S_ISREG(st.st_mode) == 0) {
		printf("Error: %s: it is not a regular file.\n", conf_filename);
		return -1;
	} else if ((st.st_uid != 0) || (st.st_gid != 0)) {
		printf("Error: the owner of %s is not root!\n", conf_filename);
		return -1;
	} else if (access(conf_filename, R_OK|W_OK|F_OK)) {
		perror(conf_filename);
		return -1;
	}

	/*
	 * Ok, all file attributes of /etc/jtager/jtager.conf are normal.
	 * so we are ready to parse it.
	 */
	max = cfgParse(conf_filename, jtager_cfg, CFG_INI);
	if (max == -1) {
		printf("%s: configuration file syntax error.\n", conf_filename);
		return max;
	}
#if 0
	printf("core-type: %s\n", core_type[0]);
	printf("emulator-type: %s\n", emulator_type[1]);
#endif	
	retval = -1;
	if (!strncasecmp(core_type[0], "arm7", 4)) {
		retval = target_setup(TARGET_TYPE_ARM7TDMI);
	} else if (!strncasecmp(core_type[0], "arm9", 4)) {
		retval = target_setup(TARGET_TYPE_ARM9TDMI);
	}
	
	if (retval)
		printf("error: failed to setup core type: %s\n", *core_type);
	else
		printf("target core type: %s\n", *core_type);

	return retval;
} /* end of setup_config() */
