/*
 * misc.c : implements some help functions for JTAGER
 *
 * Copyright (C) 2003, Rongkai Zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: misc.c,v 1.3 2004/10/17 13:54:32 zhanrk Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "jtager.h"

void reverse_bit_order(int bitnr, u8 *input, u8 *output)
{
	int input_bitcnt, input_bytecnt;
	int output_bitcnt, output_bytecnt;
	int bytenr;
	u8 input_mask , output_mask;

	if (bitnr == 0)
		return;

	bytenr = bitnr/8;
	if ((bitnr%8) != 0)
		bytenr++;
	
	/* clear output */
	for (input_bytecnt = 0; input_bytecnt < bytenr; input_bytecnt++)
		output[input_bytecnt] = 0;
	
	/* loop */
	input_bitcnt = bitnr - 1;
	input_bytecnt = bytenr - 1;
	input_mask = 0x01 << ((bitnr - 1) % 8);
	output_bitcnt = 0;
	output_bytecnt = 0;
	output_mask = 0x01;
	for ( ;output_bitcnt < bitnr; ) {
		if ((output_bitcnt % 8) == 0) {
			output_bytecnt = output_bitcnt / 8;
			output_mask = 0x01;
		}

		if (input[input_bytecnt] & input_mask)
			output[output_bytecnt] |= output_mask;
		
		/* update */
		output_bitcnt ++;
		output_mask <<= 1;

		input_bitcnt --;
		input_mask >>= 1;
		if (((input_bitcnt + 1) % 8) == 0) {
			input_bytecnt = input_bitcnt / 8;
			input_mask = 0x80;
		}
	}
} /* end of reverse_bit_order(...) */

int str2int(const char *strval, u32 *value)
{
	int i, len, retval;
	char ch, last;
	char *str, *str2;

	if (!strval)
		return -1;
	
	len = strlen(strval);
	str = malloc(len + 1);
	if (!str)
		return -1;
	strcpy(str, strval);
	
	retval = 0;
	*value = 0;
	last = *(str + len - 1);
	if ((strncasecmp("0x", str, 2) == 0) ||
		(last == 'h') || (last == 'H')) {
		/*
		 * This is a hex number.
		 */
		str2 = str;
		if (strncasecmp("0x", str2, 2) == 0)
			str2 += 2;
		if ((last == 'h') || (last == 'H'))
			*(str + len - 1) = '\0';
		
		/* how many hex digit chars */
		len = strlen(str2);
		if (len > 8) {
			retval = -1;
			goto out;
		}

		i = 0;
		while(i < len) {
			ch = tolower(str2[len-1-i]);
			
			/* check whether all chars are hex digit char */
			if (!isxdigit(ch)) {
				retval = -1;
				break;
			} else if (isdigit(ch))
				ch -= '0';
			else
				ch -= ('a' - 10);

			*value += ch << (4 * i);
			i ++;
		}
	} else {
		/*
		 * This is decimal number.
		 */
		if (len > 10) {
			retval = -1;
			goto out;
		}
		i = 0;
		while (i < len) {
			ch = str[len-1-i];
			if (!isdigit(ch)) {
				retval = -1;
				break;
			} else
				ch -= '0';
			
			*value += ch * (u32)pow(10, i);
			i++;
		}
	}
out:
	free(str);
	if (retval == -1)
		*value = 0;
	return retval;
} /* end of str2int(...) */
