/*
 * target/arm7tdmi/register.c: implements the ARM7TDMI target core's registers
 * 			       read/write operations.
 *
 * Copyright (C) 2003, Rongkai zhan <zhanrk@163.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: register.c,v 1.3 2004/10/17 13:54:32 zhanrk Exp $ */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jtager.h"
#include "jtag.h"
#include "target.h"

core_register_t arm7tdmi_regs[] = {
	{"R0", 0x0L}, {"R1", 0x0L}, {"R2", 0x0L}, {"R3", 0x0L},
	{"R4", 0x0L}, {"R5", 0x0L}, {"R6", 0x0L}, {"R7", 0x0L},
	{"R8", 0x0L}, {"R9", 0x0L}, {"R10", 0x0L}, {"R11", 0x0L},
	{"R12", 0x0L}, {"R13", 0x0L}, {"R14", 0x0L}, {"R15", 0x0L},
	{"PC", 0x0L}, {"CPSR", 0x0L}, {"SPSR", 0x0L},
	{NULL, 0x0L} /* the last element must be NULL */
};

/*
 * arm7tdmi_r0_read - read the content of R0 register
 */
u32 arm7tdmi_r0_read(void)
{
	scan_chain_t *sc1 = &arm7tdmi_target.sc[1];
	u32 value;
	
	sc1->writein[0] = 0xE5800000; /* STR R0, [R0] */
	sc1->writein[1] = DEBUG_SPEED;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "STR R0, [R0]" -- read out R0
	 * This is 1st cycle of STR instruction.
	 * In the 2nd cycle, R0's value will be visible on the data bus.
	 */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* 2nd cycle of STR instruction */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	value = sc1->readout[0]; /* get R0 value */
	return value;
} /* arm7tdmi_r0_read( ) */

/*
 * arm7tdmi_r0_write - write a new value into R0 register
 * 
 * NOTE: It seems only if we must perform two write stage,
 * the things will work. What's wrong??? Who can tell me?
 * 	 (1) Stage1: write a zero to R0 register.
 * 	 (2) Stage2: write the true new value to R0 register.
 */
void arm7tdmi_r0_write(u32 newval)
{
	scan_chain_t *sc1 = &arm7tdmi_target.sc[1];
	
	/*
	 * Stage 1: zero R0 register.
	 */
	sc1->writein[0] = 0xE5900000; /* LDR R0, [R0] = 0xE5900000 */
	sc1->writein[1] = DEBUG_SPEED;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* 1st cycle of LDR instruction */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* 2nd cycle of LDR instruction -- scan in data */
	sc1->writein[0] = 0;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* 3rd cycle of LDR instruction */
	sc1->writein[0] = ARM_NOP;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/*
	 * Stage 2: write the true new value.
	 */
	sc1->writein[0] = 0xE5900000; /* LDR R0, [R0] = 0xE5900000 */
	sc1->writein[1] = DEBUG_SPEED;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* LDR instruction's 1st cycle */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* LDR instruction's 2nd cycle ... scan in data */
	sc1->writein[0] = newval; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* LDR instruction's 3rd cycle */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	return;
} /* end of arm7tdmi_r0_write(...) */

static u32 arm7tdmi_cpsr_read(void)
{
	scan_chain_t *sc1 = &arm7tdmi_target.sc[1];
	u32 value;

	sc1->writein[0] = 0xE10F0000; /* MRS R0, CPSR */
	sc1->writein[1] = DEBUG_SPEED;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = 0xE5800000; /* STR R0, [R0] */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "MRS R0, CPSR" */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "STR R0, [R0]" -- read out R0
	 * This is 1st cycle of STR instruction.
	 * In the 2nd cycle, R0's value will be visible on the data bus.
	 */
	sc1->writein[0] = ARM_NOP;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* 2nd cycle of STR instruction */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	value = sc1->readout[0]; /* get CPSR value */
	return value;
} /* end of arm7tdmi_cpsr_read() */

static u32 arm7tdmi_spsr_read(void)
{
	scan_chain_t *sc1 = &arm7tdmi_target.sc[1];
	u32 value;

	sc1->writein[0] = 0xE14F0000; /* MRS R0, SPSR */
	sc1->writein[1] = DEBUG_SPEED;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = 0xE5800000; /* STR R0, [R0] */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "MRS R0, CPSR" */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "STR R0, [R0]" -- read out R0
	 * This is 1st cycle of STR instruction.
	 * In the 2nd cycle, R0's value will be visible on the data bus.
	 */
	sc1->writein[0] = ARM_NOP;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* 2nd cycle of STR instruction */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	value = sc1->readout[0]; /* get SPSR value */
	return value;
} /* end of arm7tdmi_spsr_read() */

/*
 * arm7tdmi_get_core_state - Read all registers of cpu core
 * 
 * NOTE: We use the following instructions sequence to determine
 * the internal state of ARM7TDMI core:
 * 	STMIA R0, {R0-R15} ; make the contents of R1-R15 registers
 * 			   ; visible on the data bus.
 * 	MRS R0, CPSR ; copy CPSR to R0
 * 	STR R0, [R0] ; make CPSR's content visible on the data bus.
 * 	MRS R0, SPSR ; copy SPSR to R0
 * 	STR R0, [R0] ; make SPSR's content visible on the data bus.
 * 		     ; At the same time, we read out the value of CPSR.
 * 	MOV R0, R0 ; NOP
 * 	MOV R0, R0 ; NOP, read out the value of SPSR.
 *
 * NOTICE: All the above instructions are executed in debug-speed.
 *
 * STMIA R0, {R0-R15} = 0xE880FFFF
 * OPCODE: cond = AL (1110), always executed.
 * 	   bit[27:25] = 100, fixed.
 * 	   P bit (bit[24]) = 0, Post: add offset after transfer.
 * 	   U bit (bit[23]) = 1, Up: add offset to base.
 * 	   S bit (bit[22]) = 0. 
 * 	   W bit (bit[21]) = 0, do not update base register.
 * 	   L bit (bit[20]) = 0, Store instruction.
 * 	   Rn (bit[19:16]) = 0000, R0 is the base register.
 * 	   Register List (bit[15:0]) = FFFF, all register is invovled.
 * 1110 1000 1000 0000 1111 1111 1111 1111 = 0xE880FFFF
 *
 * MRS R0, CPSR = 0xE10F0000
 * STR R0, [R0] = 0xE5800000
 * MRS R0, SPSR = 0xE14F0000
 * MOV R0, R0 ; NOP, OPCODE = 0xE1A00000
 */
int arm7tdmi_get_core_state(void)
{
	int i, retval;
	core_register_t *reg_pc = reg_index("PC");
	core_register_t *cpsr = reg_index("CPSR");
	core_register_t *spsr = reg_index("SPSR");
	scan_chain_t *sc1 = &target->sc[1];

	if (target->status == TARGET_STATUS_RUNNING)
		return -ERR_TARGET_IS_RUNNING;
	else if (target->mode == TARGET_MODE_THUMB)
		return -ERR_TARGET_IN_THUMB_MODE;
	
	/*
	 * Select scan chain 1, and use INTEST instruction to make scan
	 * chain 1 into the internal test mode.
	 */	
	retval = jtag_select_scanchain(1);
	if (retval)
		return retval;
	retval = jtag_write_ireg(JTAG_INTEST);
	if (retval)
		return retval;

	/* all the following instructions are executed in debug-speed */
	sc1->writein[1] = DEBUG_SPEED;
	
	/* STMIA R0, {R0-R15} = 0xE880FFFF */
	sc1->writein[0] = 0xE880FFFF;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	sc1->writein[0] = ARM_NOP;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/*
	 * Execute "STMIA R0, {R0-R15} while fetching this instruction.
	 * This is the 1st cycle of STM instruction ...
	 */
	sc1->writein[0] = ARM_NOP;
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/* 
	 * Read out the contents of r0-r15 registers in the
	 * next 16 cycles of STM instruction.
	 */
	sc1->writein[0] = ARM_NOP;
	for (i = 0; i <= 15; i++) {
		arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
		target->regs[i].value = sc1->readout[0];
	}

	/*
	 * Read CPSR and SPSR registers ...
	 */
	cpsr->value = arm7tdmi_cpsr_read();
	if ((cpsr->value & 0x1f) != 0x10) {
		spsr->value = arm7tdmi_spsr_read();
	} else {
		/* If the target is in User-Mode, the don't touch SPSR register,
		 * because there is no SPSR register in User-Mode.
		 */
		spsr->value = 0;
	}
	
	/* 
	 * restore R0 register
	 */
	arm7tdmi_r0_write(arm7tdmi_target.regs[0].value);
	
	/*
	 * Finally, correct the value of PC to the real halted point.
	 * Entry into debug state by external DBGRQ will advance
	 * the PC by three addresses or 12 bytes. Reading out the
	 * value of R15 needs to execute two instructions in debug
	 * state, which will advance the PC by two addresses or 8
	 * bytes.
	 */
	if (target->halt_reason == TARGET_HALTED_BY_DBGRQ)
		target->regs[15].value -= (3 + 2) * 4;
	else
		target->regs[15].value -= (4 + 2) * 4;
	reg_pc->value = target->regs[15].value;

	return 0;
} /* end of arm7tdmi_get_core_state(...) */

/*
 * arm7tdmi_register_read - read some specific register
 */
int arm7tdmi_register_read(core_register_t *reg)
{
	scan_chain_t *sc1 = &target->sc[1];
	u32 r0_val; /* the value of R0 */
	char *tmp;
	u32 reg_num;
	int retval;

	if (!reg || !reg->name)
		return -ERR_TARGET_UNKNOWN_REG;
	else if (target->status == TARGET_STATUS_RUNNING)
		return -ERR_TARGET_IS_RUNNING;
	else if (target->mode == TARGET_MODE_THUMB)
		return -ERR_TARGET_IN_THUMB_MODE;
	else if (strcasecmp(reg->name, "SPSR") == 0) {
		/* Only if we have an valid CPSR register value, we can
		 * read/write SPSR register value.
		 */
		if (reg_index("CPSR")->value == 0)
			return -ERR_TARGET_INVALID_CPSR;
		/* no SPSR register in user mode */
		if ((reg_index("CPSR")->value & 0x1F) == 0x10)
			return -ERR_TARGET_NO_SPSR;
	}
	
	/*
	 * Select scan chain 1, and use INTEST instruction to make scan
	 * chain 1 into the internal test mode.
	 */
	retval = jtag_select_scanchain(1);
	if (retval)
		return retval;
	retval = jtag_write_ireg(JTAG_INTEST);
	if (retval)
		return retval;

	/* all the following instructions are executed in debug-speed */
	sc1->writein[1] = DEBUG_SPEED;

	/*
	 * If the register to read is not R0, then we first
	 * save the content of R0 register.
	 */
	r0_val = arm7tdmi_r0_read();
	if (strcasecmp(reg->name, "R0") == 0) {
		/* just return directly */
		reg->value = r0_val;
		return 0;
	}
	
	/*
	 * We first copy the content of Source Register to R0.
	 * NOTE: This will clobber R0 register.
	 */
	if (strcasecmp(reg->name, "CPSR") == 0) {
		/* MRS R0, CPSR = 0xE10F0000 */
		sc1->writein[0] = 0xE10F0000;
	} else if (strcasecmp(reg->name, "SPSR") == 0) {
		/* MRS R0, SPSR = 0xE14F0000 */
		sc1->writein[0] = 0xE14F0000;
	} else if (strcasecmp(reg->name, "PC") == 0) {
		/* MOV R0, PC = 0xE1A0000F */
		sc1->writein[0] = 0xE1A0000F;
	} else { /* R1 - R15 */
		/* MOV R0, Rxx = 0xE1A0000x */
		sc1->writein[0] = 0xE1A00000;

		tmp = reg->name + 1;
		str2int(tmp, &reg_num);
		/* set the Rm bits (bit[3:0]) of the OPCODE of
		 * the "MOV R0, Rxx" instruction
		 */
		sc1->writein[0] |= (reg_num & 0x0f);
	}
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = 0xE5800000; /* STR R0, [R0] */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* Execute "STR R0, [R0]" -- read out R0
	 * This is 1st cycle of STR instruction.
	 * In the 2nd cycle, R0's value will be visible on the data bus.
	 */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/* 2nd cycle of STR instruction */
	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);
	
	/*
	 * OK, we get the value we want ...
	 */
	reg->value = sc1->readout[0];

	/*
	 * Finally, restore R0 register ...
	 */
	arm7tdmi_r0_write(r0_val);
	return 0;
} /* end of arm7tdmi_register_read(...) */

/*
 * LDR R0, [R0] = 0xE5900000
 * OPCODE: cond = AL (1110), always executed.
 * 		    bit[27:26] = 01, fixed.
 * 		    I bit (bit[25]) = 0, Offset is an immediate value.
 * 		    P bit (bit[24]) = 1, Pre: add offset before transfer.
 * 		    U bit (bit[23]) = 1, Up: Add offset to base.
 * 		    B bit (bit[22]) = 0, transfer word quantity.
 * 		    W bit(bit[21]) = 0, No write-back to base.
 * 		    L bit(bit[20]) = 1, Store instruction.
 * 		    Rn (bit[19:16]) = 0000, R0 is the Base Register.
 * 		    Rd (bit[15:12]) = 0000, R0 is the Source Register.
 * 		    Offset (bit[11:0]) = 0x0.
 * 1110 0101 1001 0000 0000 0000 0000 0000 = 0xE5900000
 */
int arm7tdmi_register_write(core_register_t *reg, u32 value)
{
	scan_chain_t *sc1 = &target->sc[1];
	u32 r0_val = 0; /* the value of R0 */
	u32 data;
	char *tmp;
	u32 reg_num;
	int retval;
	

	if (!reg || !reg->name)
		return -ERR_TARGET_UNKNOWN_REG;
	else if (target->status == TARGET_STATUS_RUNNING)
		return -ERR_TARGET_IS_RUNNING;
	else if (target->mode == TARGET_MODE_THUMB)
		return -ERR_TARGET_IN_THUMB_MODE;
	else if (strcasecmp(reg->name, "SPSR") == 0) {
		/* Only if we have an valid CPSR register value, we can
		 * read/write SPSR register value.
		 */
		if (reg_index("CPSR")->value == 0)
			return -ERR_TARGET_INVALID_CPSR;
		/* no SPSR register in user mode */
		if ((reg_index("CPSR")->value & 0x1F) == 0x10)
			return -ERR_TARGET_NO_SPSR;
	}
	
	/*
	 * Select scan chain 1, and use INTEST instruction to make scan
	 * chain 1 into the internal test mode.
	 */
	retval = jtag_select_scanchain(1);
	if (retval)
		return retval;
	retval = jtag_write_ireg(JTAG_INTEST);
	if (retval)
		return retval;

	/* all instructions are executed in debug-speed */
	sc1->writein[1] = DEBUG_SPEED;

	/*
	 * Step 1: If the register to write is not R0, then
	 * we first save the content of R0 register.
	 */
	if (strcasecmp(reg->name, "R0") != 0)
		r0_val = arm7tdmi_r0_read();

	/*
	 * Step 2: write the new value into R0 register.
	 * And if R0 is the destination register, just return directly.
	 */
	data = value;
	if (strcasecmp(reg->name, "CPSR") == 0) {
		data &= 0xF00000FF;
		data &= (~(0x00000020)); /* never change T bit (bit[5]) */
		if ((reg->value & 0x0000001F) == 0x10) {
			/* in User mode : never change the
			 * control bits of CPSR (bit[7:0])
			 */
			data &= 0xFFFFFF00;
		}
	} else if (strcasecmp(reg->name, "SPSR") == 0) {
		data &= 0xF00000FF;
		data &= (~(0x00000020)); /* never change T bit (bit[5]) */
	}
	reg->value = data; /* update our copy */

	arm7tdmi_r0_write(data); /* write new value into R0 */
	if (strcasecmp(reg->name, "R0") == 0)
		return 0;

	/*
	 * Step 3: move the value of R0 to the dest register.
	 */
	if (strcasecmp(reg->name, "CPSR") == 0) {
		/* MSR CPSR, R0 */
		sc1->writein[0] = 0xE129F000;
	} else if (strcasecmp(reg->name, "SPSR") == 0) {
		/* MSR SPSR, R0 */
		sc1->writein[0] = 0xE169F000;
	} else if (strcasecmp(reg->name, "PC") == 0) {
		/* MOV R15, R0 */
		sc1->writein[0] = 0xE1A0F000;
	} else { /* R1 - R15 */
		/* MOV Rxx, R0 */
		sc1->writein[0] = 0xE1A00000;
		tmp = reg->name + 1;
		str2int(tmp, &reg_num);
		/* set the Rd bits (bit[15:12]) of the OPCODE of
		 * the "MOV Rxx, R0" instruction
		 */
		sc1->writein[0] |= ((reg_num << 12) & 0x0000F000);
	}
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	sc1->writein[0] = ARM_NOP; /* NOP */
	arm7tdmi_exec_instruction(sc1->writein, sc1->readout);

	/*
	 * Finally, restore R0 register
	 */
	arm7tdmi_r0_write(r0_val);

	return 0;
} /* end of arm7tdmi_register_write(...) */
